@@ -61,8 +61,8 @@ struct process *process_create(ProcessSerialNumber psn)
     process->psn = psn;
     GetProcessPID(&process->psn, &process->pid);
     process->name = process_name;
-    process->terminated = false;
-    process->ns_application = workspace_application_create_running_ns_application(process);
+    __atomic_store_n(&process->terminated, false, __ATOMIC_RELEASE);
+    __atomic_store_n(&process->ns_application, workspace_application_create_running_ns_application(process), __ATOMIC_RELEASE);
     return process;
 }
 
@@ -75,7 +75,7 @@ void process_destroy(struct process *process)
 
 static PROCESS_EVENT_HANDLER(process_handler)
 {
-    struct process_manager *pm = (struct process_manager *) user_data;
+    struct process_manager *pm = context;
 
     ProcessSerialNumber psn;
     if (GetEventParameter(event, kEventParamProcessID, typeProcessSerialNumber, NULL, sizeof(psn), NULL, &psn) != noErr) {
@@ -99,23 +99,23 @@ static PROCESS_EVENT_HANDLER(process_handler)
         if (!process) return noErr;
 
         process_manager_add_process(pm, process);
-        event_loop_post(&g_event_loop, APPLICATION_LAUNCHED, process, 0, NULL);
+        event_loop_post(&g_event_loop, APPLICATION_LAUNCHED, process, 0);
     } break;
     case kEventAppTerminated: {
         struct process *process = process_manager_find_process(pm, &psn);
         if (!process) return noErr;
 
-        process->terminated = true;
+        __atomic_store_n(&process->terminated, true, __ATOMIC_RELEASE);
         process_manager_remove_process(pm, &psn);
         __asm__ __volatile__ ("" ::: "memory");
 
-        event_loop_post(&g_event_loop, APPLICATION_TERMINATED, process, 0, NULL);
+        event_loop_post(&g_event_loop, APPLICATION_TERMINATED, process, 0);
     } break;
     case kEventAppFrontSwitched: {
         struct process *process = process_manager_find_process(pm, &psn);
         if (!process) return noErr;
 
-        event_loop_post(&g_event_loop, APPLICATION_FRONT_SWITCHED, process, 0, NULL);
+        event_loop_post(&g_event_loop, APPLICATION_FRONT_SWITCHED, process, 0);
     } break;
     }
 
